#include "lut.h"

typedef struct Point {
    int x;
    int y;
    int z;
} Point;

typedef struct Vector {
    Point p1;
    Point p2;
    char shade;
} Vector;

typedef struct Shape {
    Vector* lines;
    int numLines;
    int size;
} Shape;

typedef struct Entity {
    Shape* shape;
    Point pos;
    int rot;

    Point center;
    int size;
} Entity;

/**void drawline(int x1, int y1, int x2, int y2, char shade) {
    printf("(%d, %d) to (%d, %d)\n", x1, y1, x2, y2);
}**/

void paintShape(Entity* object, Point player) {
    Vector curLine;
    //float scaleFactor = 2 / (float)(object.pos.z - player.z); //LUT, fixed point
    //float scaleFactor = pow(0.9, object.pos.z - player.z);
    //printf("1/(%d-%d)=1/(%d)=%f\n", player.z, object.pos.z, player.z - object.pos.z, scaleFactor);
    int i = 0;

    long p1x = 0;
    long p2x = 0;
    long p2y = 0;
    long p1y = 0;
    long xDelta = object->pos.x - player.x;
    long yDelta = object->pos.y - player.y;
    long zDelta = object->pos.z - player.z;
    long scaleFactor = lut[zDelta];

    long centerX = 160 + xDelta * (200 - zDelta) / 40; //xDelta/zDelta
    long centerY = 100 + yDelta * (200 - zDelta) / 40; //yDelta/zDelta

    for (; i < object->shape->numLines; i++) {
        curLine = object->shape->lines[i];


            p1x = (curLine.p1.x * scaleFactor) >> 8;
            p1y = (curLine.p1.y * scaleFactor) >> 8;
            p2x = (curLine.p2.x * scaleFactor) >> 8;
            p2y = (curLine.p2.y * scaleFactor) >> 8;

            drawline(centerX + p1x, centerY - p1y, centerX + p2x, centerY - p2y, curLine.shade);

            //drawline((centerX + (curLine.p1.x * scaleFactor)), (centerY - (curLine.p1.y * scaleFactor)), (centerX + (curLine.p2.x * scaleFactor)), (centerY - (curLine.p2.y * scaleFactor)), curLine.shade);

    }

    object->center.x = centerX;
    object->center.y = centerY;
    object->size = (object->shape->size * scaleFactor) >> 8;
    plotpixel(centerX, centerY, 0x2C);

}

Shape* loadShape(char* fileName) {
    Shape* trivial = malloc(sizeof(Shape));
    Point* points;
    char lineHeader[8];
    int curShade;
    int p1;
    int p2;
    int res;
    int curPt = 0;
    int numPoints = 0;
    int curLine = 0;
    int eof = 0;

    FILE* fp = fopen(fileName, "r");

    while (!eof) {
        res = fscanf(fp, "%s", lineHeader);
        if (res == -1) { eof = 1; break; }

        if (strcmp(lineHeader, "size") == 0) {
            fscanf(fp, "%d\n", &(trivial->size));
        }
        if (strcmp(lineHeader, "lines") == 0) {
            fscanf(fp, "%d\n", &(trivial->numLines));
            trivial->lines = malloc(trivial->numLines * sizeof(Vector));
        }
        if (strcmp(lineHeader, "points") == 0) {
            fscanf(fp, "%d\n", &(numPoints));
            points = malloc(numPoints * sizeof(Point));
        }
        if (strcmp(lineHeader, "p") == 0) {
            fscanf(fp, "%d %d\n", &(points[curPt].x), &(points[curPt].y));
            curPt++;
        }
        if (strcmp(lineHeader, "v") == 0) {
            fscanf(fp, "%d %d %d\n", &p1, &p2, &curShade);

            trivial->lines[curLine].p1 = points[p1];
            trivial->lines[curLine].p2 = points[p2];
            trivial->lines[curLine].shade = curShade;

            curLine++;
        }

    }

    return trivial;
}